import requests

def views_for_paste(pastebin_key, proxy=None):
    pastebin_url = "http://pastebin.com/api_scrape_item_meta.php"
    req_proxy = {"http": proxy} if proxy else None
    try:
        response = requests.get(pastebin_url, params={"i": pastebin_key}, proxies=req_proxy)
        if response.status_code != 200:
            return False, "HTTP response code was {}".format(response.status_code)
        paste_data = response.json()
        return True, int(paste_data[0]["hits"])
    except ValueError, value_error:
        if response.text == "Error, this is not a public paste.":
            return True, -2
        elif response.text == "Error, we cannot find this paste.":
            return True, -1
        else:
            return False, "Invalid JSON returned from {}: {}".format(response.url, response.text)
    except requests.ConnectionError, http_error:
        return False, str(http_error)
    except KeyError, _:
        return False, "Unable to find 'hits' key in webservice response {}".format(response.text)


def pastebin_keys_to_measure(host, port):
    """Fetches a list of pastebin posts to remeasure for number of views.

    Args:
        host -- a string, describing the ip or host to query for the list
                of pastebin keys to remeasure.
        port -- int, describing the port on the given host to request
                pastebin keys from.

    Returns:
        A pair of values.  The first value is a bool, describing whether
        there the request for keys was successful.

        If the first value is False (indicating an error), the second
        value will be a string, describing the error.

        Otherwise, the second value will be a list of zero or more strings,
        with each string describing the key of a pastebin post that should
        be queried for view information.
    """
    endpoint_url = "http://{}:{}/pastes-to-count-views".format(host, port)
    try:
        response = requests.get(endpoint_url).json()
        if response['error']:
            return False, "Error received from webservice: {}".format(response['msg'])
        return True, response['msg']
    except ValueError, json_error:
        return False, "Invalid JSON returned from web service: {}".format(str(json_error))
    except Exception, http_error:
        return False, str(http_error)


def _recordviews_for_paste(host, port, pastebin_key, view_count):
    """Records the number of views on the paste with the recording server.

    Args:
        host         -- a string, describing the ip or host to query for the
                        list of pastebin keys to remeasure.
        port         -- int, describing the port on the given host to request
                        pastebin keys from.
        pastebin_key -- a string, describing a pastebin post to record view
                        count information for.
        view_count   -- an int, describing the number of times a pastebin
                        file has been viewed.

    Returns:
        A tuple of two values.  The first value is a boolean description
        of whether we were able to record the number of times a pastebin
        file has been viewed with the recording server.

        If the first value is False (indicating an error) the second value
        is a string describing the error encountered.

        If the first values is True (indicating success) the second value
        is None.
    """
    endpoint_url = "http://{}:{}/record-views-for-paste".format(host, port)
    post_params = {
        "key": pastebin_key,
        "num_views": view_count
    }
    try:
        response = requests.post(endpoint_url, data=post_params).json()
        if response['error']:
            return False, "Error received from webservice: {}".format(response['msg'])
        return True, response['msg']
    except ValueError, json_error:
        return False, "Invalid JSON returned from web service: {}".format(str(json_error))
    except Exception, http_error:
        return False, str(http_error)

def record_proxied(host, port, cancel_list, work_unit):
    pastebin_key, proxy = work_unit
    return record(host, port, cancel_list, pastebin_key, proxy=proxy)

def record(host, port, cancel_list, pastebin_key, proxy=None):
    """Fetches view count information for a pastebin, and if we're able to
    fetch it, pastes it to the recording database service.

    Args:
        host         -- a string, describing the ip or host to query for the
                        list of pastebin keys to remeasure.
        port         -- int, describing the port on the given host to request
                        pastebin keys from.
        cancel_list  -- If provided, will be a list of length 1, containing
                        a boolean.  If this boolean is False, then no
                        work should be performed (ie this work has been
                        cancelled by other items in the thread pool).

                        If the current request encounters an error and it
                        wants to prevent other items in the process pool
                        from doing work, this function should set
                        `cancel_list[0] = False`.
        pastebin_key -- a string, describing a pastebin post to record view
                        count information for.

    Keyword Args:
        proxy -- An optional string, in host[:port] format, describing a
                 http proxy that should be used when querying pastebin.

    Returns:
        A tuple of two values.  The first value is a boolean description
        of whether we were able to record view count information for the
        pastebin file.

        If the first value if False (indicating an error) the second value
        is a string describing the error.  It *may also* be None, if
        the work was cancelled by other processes in the process pool.

        If the first falue is True (indicating success) the second value
        is an int, as the number of views for the paste.
    """
    if cancel_list is not None and cancel_list[0] is False:
        return False, None
    view_req_success, num_views = views_for_paste(pastebin_key, proxy=proxy)

    if not view_req_success:
        if cancel_list is not None:
            cancel_list[0] = False
        return False, num_views
    submit_success, submit_msg = _recordviews_for_paste(host, port,
                                                        pastebin_key,
                                                        num_views)
    if not submit_success:
        if cancel_list is not None:
            cancel_list[0] = False
        return False, submit_msg
    return True, num_views
