import pprint
import datetime
import doxscrapers.sites.base
from doxscrapers.types import PostTeaser, Post

class ChanDoxScraper(doxscrapers.sites.base.BaseDoxScraper):

    def __init__(self, marshaller_host, chan_host, chan_board, log_name):
        self._base_url = "{}/{}".format(chan_host, chan_board)
        super(ChanDoxScraper, self).__init__(1, marshaller_host, log_name)


    def items_in_index(self, proxy=None):
        """
        Fetches a list of PostTeaser objects, each representing a thread /
        root posting on 8chan's doxing board.

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, or a list of zero or more PostTeaser objects.
        """
        index_url = self._base_url + "/threads.json"
        api_response = self._json_for_url(index_url, proxy=proxy)
        index_items = []

        if api_response is None:
            return None

        try:
            for a_thread_group in api_response:
                for a_thread in a_thread_group["threads"]:
                    index_items.append(PostTeaser(a_thread["no"]))

        except KeyError:
            log_values = {
                "url": index_url,
                "json_string": pprint.pformat(index_items),
            }
            self._logger.error(u"Shape of JSON returned from %(url)s was not "
                               u"the expected [[{'no': int}...]] shape, "
                               u"received: %(json_string)s", log_values)
            return None

        log_params = {
            u"count": len(index_items),
            u"url": index_url,
        }
        self._logger.info(u"Successfully received %(count)s items from %(url)s",
                          log_params)
        return index_items


    def details_for_item(self, item, proxy=None):
        """
        Returns a Post object, describing the full contents of the post
        that was listed in an index of posts.

        Args:
            item -- a doxscrapers.types.PostTeaser object

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, and otherwise a doxscrapers.types.Post
            object.
        """
        try:
            self._throttler(self._req_pause)
            post_url = self._base_url + "/res/{}.json".format(item.key)
            post_request = self._request_url(post_url, proxy=proxy)
            if post_request is None:
                return None
            api_response = post_request.json()
        except ValueError:
            log_params = {
                "url": post_url,
                "response": post_request.text,
            }
            self._logger.error(u"Invalid JSON from %(url)s: %(response)s", log_params)
            return None

        post_url = self._base_url + "/res/{}.html".format(item.key)
        api_item = api_response["posts"][0]

        # Some Chan posts don't have a text component (ex its only an image).
        # In these cases, just record an empty paste body, so that we don't
        # rescrape the paste later.
        paste_text = api_item["com"] if "com" in api_item else ""

        try:
            post = Post(item.key, paste_text, post_url)
            post.added_on = datetime.datetime.fromtimestamp(api_item["time"])
            post.user = api_item["name"] if "name" in api_item else "{}:trip".format(api_item["trip"])
        except ValueError:
            log_params = {
                u"bad_time": api_item["time"],
                u"response": post_request.text,
            }
            self._logger.error(u"Invalid time, '%(bad_time)s' returned from "
                               u"API in response: %(response)s", log_params)
            return None
        except KeyError, err:
            log_params = {
                u"response": post_request.text,
                u"missing_key": err[0],
            }
            self._logger.error(u"Received JSON from chan API doesn't have the "
                               u"expected '%(missing_key)s' in response: "
                               u"%(response)s", log_params)
            return None

        return post


class EightChanBaphometDoxScraper(ChanDoxScraper):
    def __init__(self, marshaller_host):
        super(EightChanBaphometDoxScraper, self).__init__(
            marshaller_host, "https://8ch.net", "baphomet", "eightchan-baphomet")

class EightChanPolDoxScraper(ChanDoxScraper):
    def __init__(self, marshaller_host):
        super(EightChanPolDoxScraper, self).__init__(
            marshaller_host, "https://8ch.net", "pol", "eightchan-pol")

class FourChanBDoxScraper(ChanDoxScraper):
    def __init__(self, marshaller_host):
        super(FourChanBDoxScraper, self).__init__(
            marshaller_host, "https://a.4cdn.org", "b", "fourchan-b")

class FourChanPolDoxScraper(ChanDoxScraper):
    def __init__(self, marshaller_host):
        super(FourChanPolDoxScraper, self).__init__(
            marshaller_host, "https://a.4cdn.org", "pol", "fourchan-pol")
